@@ -33,20 +33,12 @@
  */
 #define CLIENT_CLEAN_UP_TIME  300
 
-/* How often we check for stale client cache (in seconds) */
-#define CLIENT_CLEAN_UP_CHECK 60
-
-struct TrackerDBusRequestHandler {
-	TrackerDBusRequestFunc new;
-	TrackerDBusRequestFunc done;
-	gpointer               user_data;
-};
-
 typedef struct {
 	gchar *sender;
 	gchar *binary;
 	gulong pid;
 	GTimeVal last_time;
+	guint clean_up_id;
 } ClientData;
 
 typedef struct {
@@ -59,57 +51,13 @@ typedef struct {
 	gboolean expect_variable_names;
 } SendAndSpliceData;
 
-static GSList *hooks;
-static gboolean block_hooks;
-
 static gboolean client_lookup_enabled;
 static DBusGConnection *freedesktop_connection;
 static DBusGProxy *freedesktop_proxy;
 static GHashTable *clients;
-static guint clients_clean_up_id;
 
 static void     client_data_free    (gpointer data);
-static gboolean clients_clean_up_cb (gpointer data);
-
-static void
-request_handler_call_for_new (guint request_id)
-{
-	GSList *l;
-
-	if (block_hooks) {
-		return;
-	}
-
-	for (l = hooks; l; l = l->next) {
-		TrackerDBusRequestHandler *handler;
-
-		handler = l->data;
-
-		if (handler->new) {
-			(handler->new)(request_id, handler->user_data);
-		}
-	}
-}
-
-static void
-request_handler_call_for_done (guint request_id)
-{
-	GSList *l;
-
-	if (block_hooks) {
-		return;
-	}
-
-	for (l = hooks; l; l = l->next) {
-		TrackerDBusRequestHandler *handler;
-
-		handler = l->data;
-
-		if (handler->done) {
-			(handler->done)(request_id, handler->user_data);
-		}
-	}
-}
+static gboolean client_clean_up_cb (gpointer data);
 
 static gboolean
 clients_init (void)
@@ -145,8 +93,6 @@ clients_init (void)
 	                                 g_str_equal,
 	                                 NULL,
 	                                 client_data_free);
-	clients_clean_up_id =
-		g_timeout_add_seconds (CLIENT_CLEAN_UP_CHECK, clients_clean_up_cb, NULL);
 
 	return TRUE;
 }
@@ -164,11 +110,6 @@ clients_shutdown (void)
 		freedesktop_connection = NULL;
 	}
 
-	if (clients_clean_up_id != 0) {
-		g_source_remove (clients_clean_up_id);
-		clients_clean_up_id = 0;
-	}
-
 	if (clients) {
 		g_hash_table_unref (clients);
 		clients = NULL;
@@ -186,6 +127,8 @@ client_data_free (gpointer data)
 		return;
 	}
 
+	g_source_remove (cd->clean_up_id);
+
 	g_free (cd->sender);
 	g_free (cd->binary);
 
@@ -249,44 +192,22 @@ client_data_new (gchar *sender)
 }
 
 static gboolean
-clients_clean_up_cb (gpointer data)
+client_clean_up_cb (gpointer data)
 {
-	GTimeVal now;
-	GHashTableIter iter;
-	gpointer key, value;
+	ClientData *cd;
 
-	g_get_current_time (&now);
+	cd = data;
 
-	g_hash_table_iter_init (&iter, clients);
-	while (g_hash_table_iter_next (&iter, &key, &value)) {
-		ClientData *cd;
-		glong diff;
-
-		cd = value;
-
-		diff = now.tv_sec - cd->last_time.tv_sec;
-
-		/* 5 Minutes */
-		if (diff >= CLIENT_CLEAN_UP_TIME) {
-			g_debug ("Removing D-Bus client data for '%s' with id:'%s'",
-			         cd->binary, cd->sender);
-			g_hash_table_iter_remove (&iter);
-		}
-	}
+	g_debug ("Removing D-Bus client data for '%s' (pid: %lu) with id:'%s'",
+	         cd->binary, cd->pid, cd->sender);
+	g_hash_table_remove (clients, cd->sender);
 
 	if (g_hash_table_size (clients) < 1) {
-		/* This must be before the clients_shutdown which will
-		 * attempt to clean up the the timeout too.
-		 */
-		clients_clean_up_id = 0;
-
-		/* Clean everything else up. */
+		/* Clean everything up. */
 		clients_shutdown ();
-
-		return FALSE;
 	}
 
-	return TRUE;
+	return FALSE;
 }
 
 static ClientData *
@@ -321,53 +242,22 @@ client_get_for_context (DBusGMethodInvocation *context)
 		g_hash_table_insert (clients, sender, cd);
 	} else {
 		g_free (sender);
+		g_source_remove (cd->clean_up_id);
 	}
 
+	cd->clean_up_id = g_timeout_add_seconds (CLIENT_CLEAN_UP_TIME, client_clean_up_cb, cd);
+
 	g_get_current_time (&cd->last_time);
 
 	return cd;
 }
 
-GValue *
-tracker_dbus_gvalue_slice_new (GType type)
-{
-	GValue *value;
-
-	value = g_slice_new0 (GValue);
-	g_value_init (value, type);
-
-	return value;
-}
-
-void
-tracker_dbus_gvalue_slice_free (GValue *value)
-{
-	g_value_unset (value);
-	g_slice_free (GValue, value);
-}
-
 GQuark
 tracker_dbus_error_quark (void)
 {
 	return g_quark_from_static_string (TRACKER_DBUS_ERROR_DOMAIN);
 }
 
-TrackerDBusData *
-tracker_dbus_data_new (const gpointer arg1,
-                       const gpointer arg2)
-{
-	TrackerDBusData *data;
-
-	data = g_new0 (TrackerDBusData, 1);
-
-	data->id = tracker_dbus_get_next_request_id ();
-
-	data->data1 = arg1;
-	data->data2 = arg2;
-
-	return data;
-}
-
 gchar **
 tracker_dbus_slist_to_strv (GSList *list)
 {
@@ -392,110 +282,6 @@ tracker_dbus_slist_to_strv (GSList *list)
 	return strv;
 }
 
-gchar **
-tracker_dbus_str_to_strv (const gchar *str)
-{
-	gchar **strv;
-
-	strv = g_new (gchar*, 2);
-	strv[0] = g_strdup (str);
-	strv[1] = NULL;
-
-	return strv;
-}
-
-gchar **
-tracker_dbus_queue_str_to_strv (GQueue *queue,
-                                gint    max)
-{
-	gchar **strv;
-	gchar  *str;
-	gint    i, j;
-	gint    length;
-
-	length = g_queue_get_length (queue);
-
-	if (max > 0) {
-		length = MIN (max, length);
-	}
-
-	strv = g_new0 (gchar*, length + 1);
-
-	for (i = 0, j = 0; i < length; i++) {
-		str = g_queue_pop_head (queue);
-
-		if (!str) {
-			break;
-		}
-
-		if (!g_utf8_validate (str, -1, NULL)) {
-			g_message ("Could not add string:'%s' to GStrv, invalid UTF-8", str);
-			g_free (str);
-			continue;
-		}
-
-		strv[j++] = str;
-	}
-
-	strv[j] = NULL;
-
-	return strv;
-}
-
-gchar **
-tracker_dbus_queue_gfile_to_strv (GQueue *queue,
-                                  gint    max)
-{
-	gchar **strv;
-	gchar  *str;
-	GFile  *file;
-	gint    i, j;
-	gint    length;
-
-	length = g_queue_get_length (queue);
-
-	if (max > 0) {
-		length = MIN (max, length);
-	}
-
-	strv = g_new0 (gchar*, length + 1);
-
-	for (i = 0, j = 0; i < length; i++) {
-		file = g_queue_pop_head (queue);
-
-		if (!file) {
-			break;
-		}
-
-		str = g_file_get_path (file);
-		g_object_unref (file);
-
-		if (!g_utf8_validate (str, -1, NULL)) {
-			g_message ("Could not add string:'%s' to GStrv, invalid UTF-8", str);
-			g_free (str);
-			continue;
-		}
-
-		strv[j++] = str;
-	}
-
-	strv[j] = NULL;
-
-	return strv;
-}
-
-void
-tracker_dbus_results_ptr_array_free (GPtrArray **ptr_array)
-{
-	if (!ptr_array || !(*ptr_array)) {
-		return;
-	}
-
-	g_ptr_array_foreach (*ptr_array, (GFunc) g_strfreev, NULL);
-	g_ptr_array_free (*ptr_array, TRUE);
-	*ptr_array = NULL;
-}
-
 guint
 tracker_dbus_get_next_request_id (void)
 {
@@ -504,32 +290,6 @@ tracker_dbus_get_next_request_id (void)
 	return request_id++;
 }
 
-TrackerDBusRequestHandler *
-tracker_dbus_request_add_hook (TrackerDBusRequestFunc new,
-                               TrackerDBusRequestFunc done,
-                               gpointer                       user_data)
-{
-	TrackerDBusRequestHandler *handler;
-
-	handler = g_slice_new0 (TrackerDBusRequestHandler);
-	handler->new = new;
-	handler->done = done;
-	handler->user_data = user_data;
-
-	hooks = g_slist_append (hooks, handler);
-
-	return handler;
-}
-
-void
-tracker_dbus_request_remove_hook (TrackerDBusRequestHandler *handler)
-{
-	g_return_if_fail (handler != NULL);
-
-	hooks = g_slist_remove (hooks, handler);
-	g_slice_free (TrackerDBusRequestHandler, handler);
-}
-
 void
 tracker_dbus_request_new (gint                   request_id,
                           DBusGMethodInvocation *context,
@@ -546,15 +306,14 @@ tracker_dbus_request_new (gint                   request_id,
 
 	cd = client_get_for_context (context);
 
-	g_debug ("<--- [%d%s%s] %s",
+	g_debug ("<--- [%d%s%s|%lu] %s",
 	         request_id,
 	         cd ? "|" : "",
 	         cd ? cd->binary : "",
+	         cd ? cd->pid : 0,
 	         str);
 
 	g_free (str);
-
-	request_handler_call_for_new (request_id);
 }
 
 void
@@ -563,14 +322,13 @@ tracker_dbus_request_success (gint                   request_id,
 {
 	ClientData *cd;
 
-	request_handler_call_for_done (request_id);
-
 	cd = client_get_for_context (context);
 
-	g_debug ("---> [%d%s%s] Success, no error given",
+	g_debug ("---> [%d%s%s|%lu] Success, no error given",
 	         request_id,
 	         cd ? "|" : "",
-	         cd ? cd->binary : "");
+	         cd ? cd->binary : "",
+	         cd ? cd->pid : 0);
 }
 
 void
@@ -584,8 +342,6 @@ tracker_dbus_request_failed (gint                    request_id,
 	gchar *str;
 	va_list args;
 
-	request_handler_call_for_done (request_id);
-
 	if (format) {
 		va_start (args, format);
 		str = g_strdup_vprintf (format, args);
@@ -601,10 +357,11 @@ tracker_dbus_request_failed (gint                    request_id,
 
 	cd = client_get_for_context (context);
 
-	g_message ("---> [%d%s%s] Failed, %s",
+	g_message ("---> [%d%s%s|%lu] Failed, %s",
 	           request_id,
 	           cd ? "|" : "",
 	           cd ? cd->binary : "",
+	           cd ? cd->pid : 0,
 	           str);
 	g_free (str);
 }
@@ -625,10 +382,11 @@ tracker_dbus_request_info (gint                   request_id,
 
 	cd = client_get_for_context (context);
 
-	tracker_info ("---- [%d%s%s] %s",
+	tracker_info ("---- [%d%s%s|%lu] %s",
 	              request_id,
 	              cd ? "|" : "",
 	              cd ? cd->binary : "",
+	              cd ? cd->pid : 0,
 	              str);
 	g_free (str);
 }
@@ -649,10 +407,11 @@ tracker_dbus_request_comment (gint                   request_id,
 
 	cd = client_get_for_context (context);
 
-	g_message ("---- [%d%s%s] %s",
+	g_message ("---- [%d%s%s|%lu] %s",
 	           request_id,
 	           cd ? "|" : "",
 	           cd ? cd->binary : "",
+	           cd ? cd->pid : 0,
 	           str);
 	g_free (str);
 }
@@ -673,26 +432,15 @@ tracker_dbus_request_debug (gint                   request_id,
 
 	cd = client_get_for_context (context);
 
-	g_debug ("---- [%d%s%s] %s",
+	g_debug ("---- [%d%s%s|%lu] %s",
 	         request_id,
 	         cd ? "|" : "",
 	         cd ? cd->binary : "",
+	         cd ? cd->pid : 0,
 	         str);
 	g_free (str);
 }
 
-void
-tracker_dbus_request_block_hooks (void)
-{
-	block_hooks = TRUE;
-}
-
-void
-tracker_dbus_request_unblock_hooks (void)
-{
-	block_hooks = FALSE;
-}
-
 void
 tracker_dbus_enable_client_lookup (gboolean enabled)
 {
