@@ -98,23 +98,12 @@ G_BEGIN_DECLS
 #define TRACKER_DBUS_PATH_EXTRACT      "/org/freedesktop/Tracker1/Extract"
 #define TRACKER_DBUS_INTERFACE_EXTRACT "org.freedesktop.Tracker1.Extract"
 
-typedef struct TrackerDBusRequestHandler TrackerDBusRequestHandler;
-
-typedef void (*TrackerDBusRequestFunc) (guint    request_id,
-                                        gpointer user_data);
-
 typedef void (*TrackerDBusSendAndSpliceCallback) (void     *buffer,
                                                   gssize    buffer_size,
                                                   GStrv     variable_names,
                                                   GError   *error,
                                                   gpointer  user_data);
 
-typedef struct {
-	guint    id;
-	gpointer data1;
-	gpointer data2;
-} TrackerDBusData;
-
 typedef enum {
 	TRACKER_DBUS_EVENTS_TYPE_ADD,
 	TRACKER_DBUS_EVENTS_TYPE_UPDATE,
@@ -128,29 +117,13 @@ typedef enum {
 } TrackerDBusError;
 
 GQuark           tracker_dbus_error_quark            (void);
-TrackerDBusData *tracker_dbus_data_new               (const gpointer              arg1,
-                                                      const gpointer              arg2);
 
 /* Utils */
-GValue *         tracker_dbus_gvalue_slice_new       (GType                       type);
-void             tracker_dbus_gvalue_slice_free      (GValue                     *value);
 gchar **         tracker_dbus_slist_to_strv          (GSList                     *list);
-gchar **         tracker_dbus_str_to_strv            (const gchar                *str);
-gchar **         tracker_dbus_queue_str_to_strv      (GQueue                     *queue,
-                                                      gint                        max);
-gchar **         tracker_dbus_queue_gfile_to_strv    (GQueue                     *queue,
-                                                      gint                        max);
-void             tracker_dbus_results_ptr_array_free (GPtrArray                 **ptr_array);
 
 /* Requests */
 guint            tracker_dbus_get_next_request_id    (void);
 
-TrackerDBusRequestHandler *
-                 tracker_dbus_request_add_hook       (TrackerDBusRequestFunc      new,
-                                                      TrackerDBusRequestFunc      done,
-                                                      gpointer                    user_data);
-void             tracker_dbus_request_remove_hook    (TrackerDBusRequestHandler  *handler);
-
 void             tracker_dbus_request_new            (gint                        request_id,
                                                       DBusGMethodInvocation      *context,
                                                       const gchar                *format,
@@ -174,8 +147,6 @@ void             tracker_dbus_request_debug          (gint
                                                       DBusGMethodInvocation      *context,
                                                       const gchar                *format,
                                                       ...);
-void             tracker_dbus_request_block_hooks    (void);
-void             tracker_dbus_request_unblock_hooks  (void);
 
 void             tracker_dbus_enable_client_lookup   (gboolean                    enable);
 
