@@ -80,6 +80,7 @@ struct _TrackerDataUpdateBufferResource {
 	const gchar *subject;
 	gint id;
 	gboolean create;
+	gboolean modified;
 	/* TrackerProperty -> GValueArray */
 	GHashTable *predicates;
 	/* string -> TrackerDataUpdateBufferTable */
@@ -162,6 +163,13 @@ static gint max_ontology_id = 0;
 static gint max_modseq = 0;
 
 static gint ensure_resource_id (const gchar *uri, gboolean    *create);
+static void cache_insert_value (const gchar *table_name,
+                                const gchar *field_name,
+                                GValue      *value,
+                                gint         graph,
+                                gboolean     multiple_values,
+                                gboolean     fts,
+                                gboolean     date_time);
 
 void
 tracker_data_add_commit_statement_callback (TrackerCommitCallback    callback,
@@ -483,6 +491,20 @@ cache_ensure_table (const gchar            *table_name,
 {
 	TrackerDataUpdateBufferTable *table;
 
+	if (!resource_buffer->modified) {
+		/* first modification of this particular resource, update tracker:modified */
+
+		GValue gvalue = { 0 };
+
+		resource_buffer->modified = TRUE;
+
+		g_value_init (&gvalue, G_TYPE_INT64);
+		g_value_set_int64 (&gvalue, tracker_data_update_get_next_modseq ());
+		cache_insert_value ("rdfs:Resource", "tracker:modified", &gvalue,
+			            0,
+			            FALSE, FALSE, FALSE);
+	}
+
 	table = g_hash_table_lookup (resource_buffer->tables, table_name);
 	if (table == NULL) {
 		table = cache_table_new (multiple_values);
@@ -1251,7 +1273,7 @@ get_property_values (TrackerProperty *property)
 	if (!resource_buffer->create) {
 		TrackerDBInterface *iface;
 		TrackerDBStatement *stmt;
-		TrackerDBResultSet *result_set = NULL;
+		TrackerDBCursor    *cursor = NULL;
 		const gchar        *table_name;
 		const gchar        *field_name;
 		GError             *error = NULL;
@@ -1267,7 +1289,7 @@ get_property_values (TrackerProperty *property)
 
 		if (stmt) {
 			tracker_db_statement_bind_int (stmt, 0, resource_buffer->id);
-			result_set = tracker_db_statement_execute (stmt, &error);
+			cursor = tracker_db_statement_start_cursor (stmt, &error);
 			g_object_unref (stmt);
 		}
 
@@ -1276,18 +1298,10 @@ get_property_values (TrackerProperty *property)
 			g_error_free (error);
 		}
 
-		/* We use a result_set instead of a cursor here because it's
-		 * possible that otherwise the cursor would remain open during
-		 * the call from delete_resource_description. In future we want
-		 * to allow having the same query open on multiple cursors,
-		 * right now we don't support this. Which is why this workaround */
-
-		if (result_set) {
-			gboolean valid = TRUE;
-
-			while (valid) {
+		if (cursor) {
+			while (tracker_db_cursor_iter_next (cursor, NULL, &error)) {
 				GValue gvalue = { 0 };
-				_tracker_db_result_set_get_value (result_set, 0, &gvalue);
+				tracker_db_cursor_get_value (cursor, 0, &gvalue);
 				if (G_VALUE_TYPE (&gvalue)) {
 					if (tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_DATETIME) {
 						gint time;
@@ -1301,9 +1315,8 @@ get_property_values (TrackerProperty *property)
 					g_value_array_append (old_values, &gvalue);
 					g_value_unset (&gvalue);
 				}
-				valid = tracker_db_result_set_iter_next (result_set);
 			}
-			g_object_unref (result_set);
+			g_object_unref (cursor);
 		}
 	}
 
@@ -1635,6 +1648,31 @@ delete_metadata_decomposed (TrackerProperty  *property,
 	return change;
 }
 
+static void
+db_delete_row (TrackerDBInterface *iface,
+               const gchar        *table_name,
+               gint                id)
+{
+	TrackerDBStatement *stmt;
+	GError *error = NULL;
+
+	stmt = tracker_db_interface_create_statement (iface, TRACKER_DB_STATEMENT_CACHE_TYPE_UPDATE, &error,
+	                                              "DELETE FROM \"%s\" WHERE ID = ?",
+	                                              table_name);
+
+	if (stmt) {
+		tracker_db_statement_bind_int (stmt, 0, id);
+		tracker_db_statement_execute (stmt, &error);
+		g_object_unref (stmt);
+	}
+
+	if (error) {
+		g_warning ("%s", error->message);
+		g_error_free (error);
+		error = NULL;
+	}
+}
+
 static void
 cache_delete_resource_type (TrackerClass *class,
                             const gchar  *graph,
@@ -1644,7 +1682,7 @@ cache_delete_resource_type (TrackerClass *class,
 	TrackerDBStatement *stmt;
 	TrackerDBResultSet *result_set = NULL;
 	TrackerProperty   **properties, *prop;
-	gboolean            found;
+	gboolean            found, direct_delete;
 	gint                i;
 	guint               p, n_props;
 	GError             *error = NULL;
@@ -1681,6 +1719,7 @@ cache_delete_resource_type (TrackerClass *class,
 	if (error) {
 		g_warning ("%s", error->message);
 		g_error_free (error);
+		error = NULL;
 	}
 
 	if (result_set) {
@@ -1696,6 +1735,10 @@ cache_delete_resource_type (TrackerClass *class,
 		g_object_unref (result_set);
 	}
 
+	/* bypass buffer if possible
+	   we need old property values with FTS */
+	direct_delete = (!HAVE_TRACKER_FTS && g_hash_table_size (resource_buffer->tables) == 0);
+
 	/* delete all property values */
 
 	properties = tracker_ontologies_get_properties (&n_props);
@@ -1717,6 +1760,14 @@ cache_delete_resource_type (TrackerClass *class,
 		table_name = tracker_property_get_table_name (prop);
 		field_name = tracker_property_get_name (prop);
 
+		if (direct_delete) {
+			if (multiple_values) {
+				db_delete_row (iface, table_name, resource_buffer->id);
+			}
+			/* single-valued property values are deleted right after the loop by deleting the row in the class table */
+			continue;
+		}
+
 		old_values = get_old_property_values (prop, NULL);
 
 		for (y = old_values->n_values - 1; y >= 0 ; y--) {
@@ -1753,7 +1804,11 @@ cache_delete_resource_type (TrackerClass *class,
 		}
 	}
 
-	cache_delete_row (class);
+	if (direct_delete) {
+		db_delete_row (iface, tracker_class_get_name (class), resource_buffer->id);
+	} else {
+		cache_delete_row (class);
+	}
 
 	if (!in_journal_replay && delete_callbacks) {
 		guint n;
@@ -1799,7 +1854,6 @@ resource_buffer_switch (const gchar *graph,
 	}
 
 	if (resource_buffer == NULL) {
-		GValue gvalue = { 0 };
 		gchar *subject_dup = NULL;
 
 		/* large INSERTs with thousands of resources could lead to
@@ -1838,12 +1892,6 @@ resource_buffer_switch (const gchar *graph,
 				graph_id = ensure_resource_id (graph, NULL);
 			}
 		}
-
-		g_value_init (&gvalue, G_TYPE_INT64);
-		g_value_set_int64 (&gvalue, tracker_data_update_get_next_modseq ());
-		cache_insert_value ("rdfs:Resource", "tracker:modified", &gvalue,
-		                    0,
-		                    FALSE, FALSE, FALSE);
 	}
 }
 
@@ -2353,277 +2401,18 @@ tracker_data_commit_transaction (GError **error)
 }
 
 void
-tracker_data_notify_transaction (void)
+tracker_data_notify_transaction (gboolean start_timer)
 {
 	if (commit_callbacks) {
 		guint n;
 		for (n = 0; n < commit_callbacks->len; n++) {
 			TrackerCommitDelegate *delegate;
 			delegate = g_ptr_array_index (commit_callbacks, n);
-			delegate->callback (delegate->user_data);
+			delegate->callback (start_timer, delegate->user_data);
 		}
 	}
 }
 
-
-static void
-format_sql_value_as_string (GString         *sql,
-                            TrackerProperty *property)
-{
-	switch (tracker_property_get_data_type (property)) {
-	case TRACKER_PROPERTY_TYPE_RESOURCE:
-		g_string_append_printf (sql, "(SELECT Uri FROM Resource WHERE ID = \"%s\")", tracker_property_get_name (property));
-		break;
-	case TRACKER_PROPERTY_TYPE_INTEGER:
-	case TRACKER_PROPERTY_TYPE_DOUBLE:
-		g_string_append_printf (sql, "CAST (\"%s\" AS TEXT)", tracker_property_get_name (property));
-		break;
-	case TRACKER_PROPERTY_TYPE_BOOLEAN:
-		g_string_append_printf (sql, "CASE \"%s\" WHEN 1 THEN 'true' WHEN 0 THEN 'false' ELSE NULL END", tracker_property_get_name (property));
-		break;
-	case TRACKER_PROPERTY_TYPE_DATETIME:
-		g_string_append_printf (sql, "strftime (\"%%Y-%%m-%%dT%%H:%%M:%%SZ\", \"%s\", \"unixepoch\")", tracker_property_get_name (property));
-		break;
-	default:
-		g_string_append_printf (sql, "\"%s\"", tracker_property_get_name (property));
-		break;
-	}
-}
-
-/**
- * Removes the description of a resource (embedded metadata), but keeps
- * annotations (non-embedded/user metadata) stored about the resource.
- */
-void
-tracker_data_delete_resource_description (const gchar *graph,
-                                          const gchar *url,
-                                          GError **error)
-{
-	TrackerDBInterface *iface;
-	TrackerDBStatement *stmt;
-	TrackerDBResultSet *result_set = NULL, *single_result, *multi_result = NULL;
-	TrackerClass       *class;
-	gchar              *urn;
-	GString            *sql;
-	TrackerProperty   **properties, *property;
-	int                 i;
-	gboolean            first, bail_out = FALSE;
-	gint                resource_id;
-	guint               p, n_props;
-	GError             *actual_error = NULL;
-
-	/* We use result_sets instead of cursors here because it's possible
-	 * that otherwise the query of the outer cursor would be reused by the
-	 * cursors of the inner queries. */
-
-	iface = tracker_db_manager_get_db_interface ();
-
-	/* DROP GRAPH <url> - url here is nie:url */
-
-	stmt = tracker_db_interface_create_statement (iface, TRACKER_DB_STATEMENT_CACHE_TYPE_SELECT, &actual_error,
-	                                              "SELECT ID, (SELECT Uri FROM Resource WHERE ID = \"nie:DataObject\".ID) FROM \"nie:DataObject\" WHERE \"nie:DataObject\".\"nie:url\" = ?");
-
-	if (stmt) {
-		tracker_db_statement_bind_text (stmt, 0, url);
-		result_set = tracker_db_statement_execute (stmt, &actual_error);
-		g_object_unref (stmt);
-	}
-
-	if (actual_error) {
-		g_propagate_error (error, actual_error);
-		return;
-	}
-
-	if (result_set) {
-		tracker_db_result_set_get (result_set, 0, &resource_id, -1);
-		tracker_db_result_set_get (result_set, 1, &urn, -1);
-		g_object_unref (result_set);
-	} else {
-		/* For fallback to the old behaviour, we could do this here:
-		 * resource_id = tracker_data_query_resource_id (url); */
-		return;
-	}
-
-	properties = tracker_ontologies_get_properties (&n_props);
-
-	stmt = tracker_db_interface_create_statement (iface, TRACKER_DB_STATEMENT_CACHE_TYPE_SELECT, &actual_error,
-	                                              "SELECT (SELECT Uri FROM Resource WHERE ID = \"rdf:type\") FROM \"rdfs:Resource_rdf:type\" WHERE ID = ?");
-
-	if (stmt) {
-		tracker_db_statement_bind_int (stmt, 0, resource_id);
-		result_set = tracker_db_statement_execute (stmt, &actual_error);
-		g_object_unref (stmt);
-	}
-
-	if (actual_error) {
-		g_propagate_error (error, actual_error);
-		return;
-	}
-
-	if (result_set) {
-		do {
-			gchar *class_uri;
-
-			tracker_db_result_set_get (result_set, 0, &class_uri, -1);
-
-			class = tracker_ontologies_get_class_by_uri (class_uri);
-
-			if (class == NULL) {
-				g_warning ("Class '%s' not found in the ontology", class_uri);
-				g_free (class_uri);
-				continue;
-			}
-			g_free (class_uri);
-
-			/* retrieve single value properties for current class */
-
-			sql = g_string_new ("SELECT ");
-
-			first = TRUE;
-
-			for (p = 0; p < n_props; p++) {
-				property = properties[p];
-
-				if (tracker_property_get_domain (property) == class) {
-					if (!tracker_property_get_embedded (property)) {
-						continue;
-					}
-
-					if (!tracker_property_get_multiple_values (property)) {
-						if (!first) {
-							g_string_append (sql, ", ");
-						}
-						first = FALSE;
-
-						format_sql_value_as_string (sql, property);
-					}
-				}
-			}
-
-			single_result = NULL;
-			if (!first) {
-				g_string_append_printf (sql, " FROM \"%s\" WHERE ID = ?", tracker_class_get_name (class));
-				stmt = tracker_db_interface_create_statement (iface, TRACKER_DB_STATEMENT_CACHE_TYPE_SELECT,
-				                                              &actual_error, "%s", sql->str);
-
-				if (stmt) {
-					tracker_db_statement_bind_int (stmt, 0, resource_id);
-					single_result = tracker_db_statement_execute (stmt, &actual_error);
-					g_object_unref (stmt);
-				}
-
-				if (actual_error) {
-					g_propagate_error (error, actual_error);
-					bail_out = TRUE;
-					break;
-				}
-			}
-
-			g_string_free (sql, TRUE);
-
-			i = 0;
-			for (p = 0; p < n_props; p++) {
-				property = properties[p];
-
-				if (tracker_property_get_domain (property) != class) {
-					continue;
-				}
-
-				if (!tracker_property_get_embedded (property)) {
-					continue;
-				}
-
-				if (property == tracker_ontologies_get_rdf_type ()) {
-					/* Do not delete rdf:type statements */
-					continue;
-				}
-
-				if (!tracker_property_get_multiple_values (property)) {
-					gchar *value;
-
-					/* single value property, value in single_result_set */
-
-					tracker_db_result_set_get (single_result, i++, &value, -1);
-
-					if (value) {
-						tracker_data_delete_statement (graph, urn,
-						                               tracker_property_get_uri (property),
-						                               value,
-						                               &actual_error);
-						if (actual_error) {
-							g_propagate_error (error, actual_error);
-							bail_out = TRUE;
-							break;
-						}
-						g_free (value);
-					}
-
-				} else {
-					/* multi value property, retrieve values from DB */
-
-					sql = g_string_new ("SELECT ");
-
-					format_sql_value_as_string (sql, property);
-
-					g_string_append_printf (sql,
-					                        " FROM \"%s\" WHERE ID = ?",
-					                        tracker_property_get_table_name (property));
-
-					stmt = tracker_db_interface_create_statement (iface, TRACKER_DB_STATEMENT_CACHE_TYPE_SELECT,
-					                                              &actual_error, "%s", sql->str);
-
-					if (stmt) {
-						tracker_db_statement_bind_int (stmt, 0, resource_id);
-						multi_result = tracker_db_statement_execute (stmt, NULL);
-						g_object_unref (stmt);
-					}
-
-					if (actual_error) {
-						g_propagate_error (error, actual_error);
-						bail_out = TRUE;
-						break;
-					}
-
-					if (multi_result) {
-						do {
-							gchar *value;
-
-							tracker_db_result_set_get (multi_result, 0, &value, -1);
-
-							tracker_data_delete_statement (graph, urn,
-							                               tracker_property_get_uri (property),
-							                               value,
-							                               &actual_error);
-
-							g_free (value);
-
-							if (actual_error) {
-								g_propagate_error (error, actual_error);
-								bail_out = TRUE;
-								break;
-							}
-
-						} while (tracker_db_result_set_iter_next (multi_result));
-
-						g_object_unref (multi_result);
-					}
-
-					g_string_free (sql, TRUE);
-				}
-			}
-
-			if (!first) {
-				g_object_unref (single_result);
-			}
-
-		} while (!bail_out && tracker_db_result_set_iter_next (result_set));
-
-		g_object_unref (result_set);
-	}
-
-	g_free (urn);
-}
-
 void
 tracker_data_rollback_transaction (void)
 {
@@ -2646,7 +2435,7 @@ tracker_data_rollback_transaction (void)
 		for (n = 0; n < rollback_callbacks->len; n++) {
 			TrackerCommitDelegate *delegate;
 			delegate = g_ptr_array_index (rollback_callbacks, n);
-			delegate->callback (delegate->user_data);
+			delegate->callback (TRUE, delegate->user_data);
 		}
 	}
 }
