@@ -45,6 +45,8 @@ struct _TrackerPropertyPrivate {
 	gchar         *name;
 	gchar         *table_name;
 
+	gboolean       use_gvdb;
+
 	TrackerPropertyType  data_type;
 	TrackerClass   *domain;
 	TrackerClass   *domain_index;
@@ -55,7 +57,6 @@ struct _TrackerPropertyPrivate {
 	TrackerProperty *secondary_index;
 	gboolean       fulltext_indexed;
 	gboolean       fulltext_no_limit;
-	gboolean       embedded;
 	gboolean       multiple_values;
 	gboolean       last_multiple_values;
 	gboolean       transient;
@@ -133,7 +134,6 @@ tracker_property_init (TrackerProperty *property)
 
 	priv->id = 0;
 	priv->weight = 1;
-	priv->embedded = TRUE;
 	priv->transient = FALSE;
 	priv->multiple_values = TRUE;
 	priv->super_properties = g_array_new (TRUE, TRUE, sizeof (TrackerProperty *));
@@ -186,12 +186,18 @@ property_finalize (GObject *object)
  * Returns: The newly created #TrackerProperty
  **/
 TrackerProperty *
-tracker_property_new (void)
+tracker_property_new (gboolean use_gvdb)
 {
 	TrackerProperty *property;
+	TrackerPropertyPrivate *priv;
 
 	property = g_object_new (TRACKER_TYPE_PROPERTY, NULL);
 
+	if (use_gvdb) {
+		priv = GET_PRIV (property);
+		priv->use_gvdb = use_gvdb;
+	}
+
 	return property;
 }
 
@@ -242,12 +248,12 @@ tracker_property_get_table_name (TrackerProperty *property)
 	priv = GET_PRIV (property);
 
 	if (!priv->table_name) {
-		if (priv->multiple_values) {
+		if (tracker_property_get_multiple_values (property)) {
 			priv->table_name = g_strdup_printf ("%s_%s",
-				tracker_class_get_name (priv->domain),
-				priv->name);
+				tracker_class_get_name (tracker_property_get_domain (property)),
+				tracker_property_get_name (property));
 		} else {
-			priv->table_name = g_strdup (tracker_class_get_name (priv->domain));
+			priv->table_name = g_strdup (tracker_class_get_name (tracker_property_get_domain (property)));
 		}
 	}
 
@@ -263,6 +269,27 @@ tracker_property_get_data_type (TrackerProperty *property)
 
 	priv = GET_PRIV (property);
 
+	if (priv->use_gvdb) {
+		const gchar *range_uri;
+
+		range_uri = tracker_ontologies_get_property_string_gvdb (priv->uri, "range");
+		if (strcmp (range_uri, XSD_STRING) == 0) {
+			priv->data_type = TRACKER_PROPERTY_TYPE_STRING;
+		} else if (strcmp (range_uri, XSD_BOOLEAN) == 0) {
+			priv->data_type = TRACKER_PROPERTY_TYPE_BOOLEAN;
+		} else if (strcmp (range_uri, XSD_INTEGER) == 0) {
+			priv->data_type = TRACKER_PROPERTY_TYPE_INTEGER;
+		} else if (strcmp (range_uri, XSD_DOUBLE) == 0) {
+			priv->data_type = TRACKER_PROPERTY_TYPE_DOUBLE;
+		} else if (strcmp (range_uri, XSD_DATE) == 0) {
+			priv->data_type = TRACKER_PROPERTY_TYPE_DATE;
+		} else if (strcmp (range_uri, XSD_DATETIME) == 0) {
+			priv->data_type = TRACKER_PROPERTY_TYPE_DATETIME;
+		} else {
+			priv->data_type = TRACKER_PROPERTY_TYPE_RESOURCE;
+		}
+	}
+
 	return priv->data_type;
 }
 
@@ -278,6 +305,10 @@ tracker_property_get_domain (TrackerProperty *property)
 
 	priv = GET_PRIV (property);
 
+	if (!priv->domain && priv->use_gvdb) {
+		priv->domain = tracker_ontologies_get_class_by_uri (tracker_ontologies_get_property_string_gvdb (priv->uri, "domain"));
+	}
+
 	return priv->domain;
 }
 
@@ -293,6 +324,29 @@ tracker_property_get_domain_indexes (TrackerProperty *property)
 
 	priv = GET_PRIV (property);
 
+	if (priv->use_gvdb) {
+		TrackerClass *domain_index;
+		GVariant *variant;
+
+		tracker_property_reset_domain_indexes (property);
+
+		variant = tracker_ontologies_get_property_value_gvdb (priv->uri, "domain-indexes");
+		if (variant) {
+			GVariantIter iter;
+			gchar *uri;
+
+			g_variant_iter_init (&iter, variant);
+			while (g_variant_iter_loop (&iter, "s", &uri)) {
+				domain_index = tracker_ontologies_get_class_by_uri (uri);
+				g_free (uri);
+
+				tracker_property_add_domain_index (property, domain_index);
+
+				g_object_unref (domain_index);
+			}
+		}
+	}
+
 	return (TrackerClass ** ) priv->domain_indexes->data;
 }
 
@@ -305,6 +359,10 @@ tracker_property_get_range (TrackerProperty *property)
 
 	priv = GET_PRIV (property);
 
+	if (!priv->range && priv->use_gvdb) {
+		priv->range = tracker_ontologies_get_class_by_uri (tracker_ontologies_get_property_string_gvdb (priv->uri, "range"));
+	}
+
 	return priv->range;
 }
 
@@ -444,18 +502,6 @@ tracker_property_get_db_schema_changed (TrackerProperty *property)
 	return priv->db_schema_changed;
 }
 
-gboolean
-tracker_property_get_embedded (TrackerProperty *property)
-{
-	TrackerPropertyPrivate *priv;
-
-	g_return_val_if_fail (TRACKER_IS_PROPERTY (property), FALSE);
-
-	priv = GET_PRIV (property);
-
-	return priv->embedded;
-}
-
 gboolean
 tracker_property_get_multiple_values (TrackerProperty *property)
 {
@@ -465,6 +511,21 @@ tracker_property_get_multiple_values (TrackerProperty *property)
 
 	priv = GET_PRIV (property);
 
+	if (priv->use_gvdb) {
+		GVariant *value;
+		gboolean result;
+
+		value = tracker_ontologies_get_property_value_gvdb (priv->uri, "max-cardinality");
+		if (value != NULL) {
+			result = FALSE;
+			g_variant_unref (value);
+		} else {
+			result = TRUE;
+		}
+
+		return result;
+	}
+
 	return priv->multiple_values;
 }
 
@@ -489,6 +550,21 @@ tracker_property_get_is_inverse_functional_property (TrackerProperty *property)
 
 	priv = GET_PRIV (property);
 
+	if (priv->use_gvdb) {
+		GVariant *value;
+		gboolean result;
+
+		value = tracker_ontologies_get_property_value_gvdb (priv->uri, "inverse-functional");
+		if (value != NULL) {
+			result = g_variant_get_boolean (value);
+			g_variant_unref (value);
+		} else {
+			result = FALSE;
+		}
+
+		return result;
+	}
+
 	return priv->is_inverse_functional_property;
 }
 
@@ -851,19 +927,6 @@ tracker_property_set_fulltext_no_limit (TrackerProperty *property,
 	priv->fulltext_no_limit = value;
 }
 
-void
-tracker_property_set_embedded (TrackerProperty *property,
-                               gboolean         value)
-{
-	TrackerPropertyPrivate *priv;
-
-	g_return_if_fail (TRACKER_IS_PROPERTY (property));
-
-	priv = GET_PRIV (property);
-
-	priv->embedded = value;
-}
-
 void
 tracker_property_set_multiple_values (TrackerProperty *property,
                                       gboolean         value)
