@@ -19,6 +19,8 @@
 
 #include "config.h"
 
+#include <math.h>
+
 #include <libtracker-common/tracker-dbus.h>
 #include <libtracker-common/tracker-type-utils.h>
 
@@ -28,6 +30,15 @@
 #include "tracker-miner-glue.h"
 #include "tracker-dbus.h"
 
+/* Here we use ceil() to eliminate decimal points beyond what we're
+ * interested in, which is 2 decimal places for the progress. The
+ * ceil() call will also round up the last decimal place.
+ *
+ * The 0.49 value is used for rounding correctness, because ceil()
+ * rounds up if the number is > 0.0.
+ */
+#define PROGRESS_ROUNDED(x) (ceil (((x) * 100) - 0.49) / 100)
+
 /**
  * SECTION:tracker-miner
  * @short_description: Abstract base class for data miners
@@ -96,9 +107,9 @@ static void       miner_constructed            (GObject       *object);
 static void       pause_data_destroy           (gpointer       data);
 static PauseData *pause_data_new               (const gchar   *application,
                                                 const gchar   *reason);
-static void       store_name_monitor_cb (TrackerMiner *miner,
-                                         const gchar  *name,
-                                         gboolean      available);
+static void       store_name_monitor_cb        (TrackerMiner  *miner,
+                                                const gchar   *name,
+                                                gboolean       available);
 
 G_DEFINE_ABSTRACT_TYPE (TrackerMiner, tracker_miner, G_TYPE_OBJECT)
 
@@ -255,7 +266,7 @@ tracker_miner_init (TrackerMiner *miner)
 
 	miner->private = priv = TRACKER_MINER_GET_PRIVATE (miner);
 
-	priv->connection = tracker_sparql_connection_get (&error);
+	priv->connection = tracker_sparql_connection_get (NULL, &error);
 	g_assert_no_error (error);
 
 	priv->pauses = g_hash_table_new_full (g_direct_hash,
@@ -303,10 +314,15 @@ miner_set_property (GObject      *object,
 	case PROP_PROGRESS: {
 		gdouble new_progress;
 
-		new_progress = g_value_get_double (value);
+		new_progress = PROGRESS_ROUNDED (g_value_get_double (value));
 
-		/* Only notify 1% changes */
-		if ((gint) (miner->private->progress * 100) == (gint) (new_progress * 100)) {
+		/* NOTE: We don't round the current progress before
+		 * comparison because we use the rounded value when
+		 * we set it last.
+		 *
+		 * Only notify 1% changes
+		 */ 
+		if (new_progress == miner->private->progress) {
 			/* Same, do nothing */
 			break;
 		}
@@ -359,6 +375,9 @@ miner_finalize (GObject *object)
 
 	g_hash_table_unref (miner->private->pauses);
 
+	_tracker_miner_dbus_remove_name_watch (miner,
+	                                       "org.freedesktop.Tracker1",
+	                                       store_name_monitor_cb);
 	_tracker_miner_dbus_shutdown (miner);
 
 	G_OBJECT_CLASS (tracker_miner_parent_class)->finalize (object);
@@ -370,8 +389,9 @@ miner_constructed (GObject *object)
 	TrackerMiner *miner = TRACKER_MINER (object);
 
 	_tracker_miner_dbus_init (miner, &dbus_glib__tracker_miner_object_info);
-	_tracker_miner_dbus_add_name_watch (miner, "org.freedesktop.Tracker1",
-                                            store_name_monitor_cb);
+	_tracker_miner_dbus_add_name_watch (miner, 
+	                                    "org.freedesktop.Tracker1",
+	                                    store_name_monitor_cb);
 }
 
 static void
@@ -381,8 +401,9 @@ store_name_monitor_cb (TrackerMiner *miner,
 {
 	GError *error = NULL;
 
-	g_debug ("Store availability has changed to %s",
-		 available ? "AVAILABLE" : "UNAVAILABLE");
+	g_debug ("Miner:'%s' noticed store availability has changed to %s",
+	         miner->private->name,
+	         available ? "AVAILABLE" : "UNAVAILABLE");
 
 	if (available && miner->private->availability_cookie != 0) {
 		tracker_miner_resume (miner,
@@ -551,7 +572,7 @@ tracker_miner_pause_internal (TrackerMiner  *miner,
 
 	if (g_hash_table_size (miner->private->pauses) == 1) {
 		/* Pause */
-		g_message ("Miner is pausing");
+		g_message ("Miner:'%s' is pausing", miner->private->name);
 		g_signal_emit (miner, signals[PAUSED], 0);
 	}
 
@@ -616,7 +637,7 @@ tracker_miner_resume (TrackerMiner  *miner,
 
 	if (g_hash_table_size (miner->private->pauses) == 0) {
 		/* Resume */
-		g_message ("Miner is resuming");
+		g_message ("Miner:'%s' is resuming", miner->private->name);
 		g_signal_emit (miner, signals[RESUMED], 0);
 	}
 
