@@ -31,7 +31,7 @@
 
 #include <gst/gst.h>
 #include <gst/tag/tag.h>
-
+#include <libtracker-common/tracker-common.h>
 #include <libtracker-extract/tracker-extract.h>
 
 #include "tracker-albumart.h"
@@ -88,6 +88,7 @@ typedef enum {
 	EXTRACT_MIME_VIDEO,
 	EXTRACT_MIME_IMAGE,
 	EXTRACT_MIME_GUESS,
+	EXTRACT_MIME_SVG,
 } ExtractMime;
 
 typedef struct {
@@ -132,6 +133,9 @@ static void extract_gstreamer_video (const gchar          *uri,
 static void extract_gstreamer_image (const gchar          *uri,
                                      TrackerSparqlBuilder *preupdate,
                                      TrackerSparqlBuilder *metadata);
+static void extract_gstreamer_svg   (const gchar          *uri,
+                                     TrackerSparqlBuilder *preupdate,
+                                     TrackerSparqlBuilder *metadata);
 static void extract_gstreamer_guess (const gchar          *uri,
                                      TrackerSparqlBuilder *preupdate,
                                      TrackerSparqlBuilder *metadata);
@@ -141,6 +145,7 @@ static TrackerExtractData data[] = {
 	{ "video/*", extract_gstreamer_video },
 	{ "image/*", extract_gstreamer_image },
 	/* Tell gstreamer to guess if mimetype guessing returns video also for audio files */
+	{ "image/svg+xml", extract_gstreamer_svg },
 	{ "video/3gpp", extract_gstreamer_guess },
 	{ "video/mp4", extract_gstreamer_guess },
 	{ "video/x-ms-asf", extract_gstreamer_guess },
@@ -282,34 +287,40 @@ add_y_date_gst_tag (TrackerSparqlBuilder  *metadata,
                     GstTagList            *tag_list,
                     const gchar           *tag)
 {
-	GDate    *date;
-	gboolean  ret;
+	GDate *date;
+	gchar buf[25];
 
 	date = NULL;
-	ret = gst_tag_list_get_date (tag_list, tag, &date);
+	buf[0] = '\0';
+
+	if (gst_tag_list_get_date (tag_list, tag, &date)) {
+		gboolean ret;
 
-	if (ret) {
 		if (date && g_date_valid (date)) {
 			if (date->julian)
 				ret = g_date_valid_julian (date->julian_days);
 			if (date->dmy)
 				ret = g_date_valid_dmy (date->day, date->month, date->year);
-		} else
+		} else {
 			ret = FALSE;
-	}
+		}
 
-	if (ret) {
-		gchar buf[25];
-
-		if (g_date_strftime (buf, 25, "%Y-%m-%dT%H:%M:%S%z", date)) {
-			tracker_sparql_builder_predicate (metadata, key);
-			tracker_sparql_builder_object_unvalidated (metadata, buf);
+		if (ret) {
+			if (g_date_strftime (buf, 25, "%Y-%m-%dT%H:%M:%S%z", date)) {
+				tracker_sparql_builder_predicate (metadata, key);
+				tracker_sparql_builder_object_unvalidated (metadata, buf);
+			}
 		}
 	}
 
 	if (date) {
 		g_date_free (date);
 	}
+
+	tracker_guarantee_date_from_file_mtime (metadata,
+	                                            key,
+	                                            buf,
+	                                            uri);
 }
 
 static void
@@ -426,13 +437,13 @@ extract_stream_metadata_tagreadbin (MetadataExtractor     *extractor,
                                     const gchar           *uri,
                                     TrackerSparqlBuilder  *metadata)
 {
-	if (extractor->mime != EXTRACT_MIME_IMAGE) {
+	if (extractor->mime != EXTRACT_MIME_IMAGE && extractor->mime != EXTRACT_MIME_SVG) {
 		add_int_gst_tag (metadata, uri, "nfo:channels", extractor->tagcache, GST_TAG_CHANNEL);
 		add_int_gst_tag (metadata, uri, "nfo:sampleRate", extractor->tagcache, GST_TAG_RATE);
 		add_time_gst_tag (metadata, uri, "nfo:duration", extractor->tagcache, GST_TAG_DURATION);
 	}
 
-	if (extractor->mime == EXTRACT_MIME_IMAGE || extractor->mime == EXTRACT_MIME_VIDEO) {
+	if (extractor->mime == EXTRACT_MIME_IMAGE || extractor->mime == EXTRACT_MIME_SVG || extractor->mime == EXTRACT_MIME_VIDEO) {
 		add_fraction_gst_tag (metadata, uri, "nfo:aspectRatio", extractor->tagcache, GST_TAG_PIXEL_RATIO);
 	}
 
@@ -449,7 +460,7 @@ extract_stream_metadata_decodebin (MetadataExtractor *extractor,
                                    const gchar       *uri,
                                    TrackerSparqlBuilder         *metadata)
 {
-	if (extractor->mime != EXTRACT_MIME_IMAGE) {
+	if (extractor->mime != EXTRACT_MIME_IMAGE && extractor->mime != EXTRACT_MIME_SVG) {
 
 		if (extractor->audio_channels >= 0) {
 			add_uint_info (metadata, uri, "nfo:channels", extractor->audio_channels);
@@ -511,6 +522,7 @@ extract_metadata (MetadataExtractor      *extractor,
 		gchar *composer_uri = NULL;
 		gchar *album_uri = NULL;
 		gchar *album_disc_uri = NULL;
+		gchar *s;
 
 		if (extractor->mime == EXTRACT_MIME_GUESS) {
 			gchar *video_codec = NULL, *audio_codec = NULL;
@@ -749,17 +761,29 @@ extract_metadata (MetadataExtractor      *extractor,
 			tracker_sparql_builder_object (metadata, "nmm:Video");
 		} else if (!needs_audio) {
 			tracker_sparql_builder_object (metadata, "nfo:Image");
-			tracker_sparql_builder_object (metadata, "nmm:Photo");
+
+			if (extractor->mime != EXTRACT_MIME_SVG) {
+				tracker_sparql_builder_object (metadata, "nmm:Photo");
+			} else {
+				tracker_sparql_builder_object (metadata, "nfo:VectorImage");
+			}
 		}
 
 		gst_tag_list_get_string (extractor->tagcache, GST_TAG_GENRE, &genre);
-		if (g_strcmp0 (genre, "Unknown") != 0) {
+		if (genre && g_strcmp0 (genre, "Unknown") != 0) {
 			tracker_sparql_builder_predicate (metadata, "nfo:genre");
 			tracker_sparql_builder_object_unvalidated (metadata, genre);
 		}
 		g_free (genre);
 
-		add_string_gst_tag (metadata, uri, "nie:title", extractor->tagcache, GST_TAG_TITLE);
+		s = NULL;
+		gst_tag_list_get_string (extractor->tagcache, GST_TAG_TITLE, &s);
+		tracker_guarantee_title_from_file (metadata,
+		                                       "nie:title",
+		                                       s,
+		                                       uri);
+		g_free (s);
+
 		add_string_gst_tag (metadata, uri, "nie:copyright", extractor->tagcache, GST_TAG_COPYRIGHT);
 		add_string_gst_tag (metadata, uri, "nie:license", extractor->tagcache, GST_TAG_LICENSE);
 		add_string_gst_tag (metadata, uri, "dc:coverage", extractor->tagcache, GST_TAG_LOCATION);
@@ -876,7 +900,12 @@ extract_metadata (MetadataExtractor      *extractor,
 			tracker_sparql_builder_object (metadata, "nmm:Video");
 		} else if (!needs_audio) {
 			tracker_sparql_builder_object (metadata, "nfo:Image");
-			tracker_sparql_builder_object (metadata, "nmm:Photo");
+
+			if (extractor->mime != EXTRACT_MIME_SVG) {
+				tracker_sparql_builder_object (metadata, "nmm:Photo");
+			} else {
+				tracker_sparql_builder_object (metadata, "nfo:VectorImage");
+			}
 		}
 	}
 
@@ -1319,6 +1348,14 @@ extract_gstreamer_image (const gchar          *uri,
 	tracker_extract_gstreamer (uri, preupdate, metadata, EXTRACT_MIME_IMAGE);
 }
 
+static void
+extract_gstreamer_svg (const gchar          *uri,
+                       TrackerSparqlBuilder *preupdate,
+                       TrackerSparqlBuilder *metadata)
+{
+	tracker_extract_gstreamer (uri, preupdate, metadata, EXTRACT_MIME_SVG);
+}
+
 static void
 extract_gstreamer_guess (const gchar          *uri,
                         TrackerSparqlBuilder *preupdate,
