@@ -643,7 +643,8 @@ on_emit_signals (gpointer user_data)
 }
 
 static void
-on_statements_committed (gpointer user_data)
+on_statements_committed (gboolean start_timer,
+                         gpointer user_data)
 {
 	TrackerResources *resources = user_data;
 	TrackerResourcesPrivate *priv;
@@ -662,7 +663,7 @@ on_statements_committed (gpointer user_data)
 		tracker_class_transact_events (class);
 	}
 
-	if (priv->signal_timeout == 0) {
+	if (start_timer && priv->signal_timeout == 0) {
 		priv->signal_timeout = g_timeout_add_seconds (TRACKER_SIGNALS_SECONDS_PER_EMIT,
 		                                              on_emit_signals,
 		                                              user_data);
@@ -674,7 +675,8 @@ on_statements_committed (gpointer user_data)
 
 
 static void
-on_statements_rolled_back (gpointer user_data)
+on_statements_rolled_back (gboolean start_timer,
+                           gpointer user_data)
 {
 	tracker_events_reset_pending ();
 	tracker_writeback_reset_pending ();
@@ -750,7 +752,7 @@ on_statement_deleted (gint         graph_id,
 }
 
 void
-tracker_resources_prepare (TrackerResources *object)
+tracker_resources_enable_signals (TrackerResources *object)
 {
 	TrackerResourcesPrivate *priv;
 
@@ -760,11 +762,10 @@ tracker_resources_prepare (TrackerResources *object)
 	tracker_data_add_delete_statement_callback (on_statement_deleted, object);
 	tracker_data_add_commit_statement_callback (on_statements_committed, object);
 	tracker_data_add_rollback_statement_callback (on_statements_rolled_back, object);
-
 }
 
-static void
-tracker_resources_finalize (GObject      *object)
+void
+tracker_resources_disable_signals (TrackerResources *object)
 {
 	TrackerResourcesPrivate *priv;
 
@@ -778,6 +779,16 @@ tracker_resources_finalize (GObject      *object)
 	if (priv->signal_timeout != 0) {
 		g_source_remove (priv->signal_timeout);
 	}
+}
+
+static void
+tracker_resources_finalize (GObject *object)
+{
+	TrackerResourcesPrivate *priv;
+
+	priv = TRACKER_RESOURCES_GET_PRIVATE (object);
+
+	tracker_resources_disable_signals ((TrackerResources *) object);
 
 	dbus_connection_unref (priv->connection);
 
