@@ -23,9 +23,9 @@
 
 #include "tracker-writeback-file.h"
 
-static gboolean tracker_writeback_file_update_metadata (TrackerWriteback *writeback,
-                                                        GPtrArray        *values,
-                                                        TrackerClient    *client);
+static gboolean tracker_writeback_file_update_metadata (TrackerWriteback        *writeback,
+                                                        GPtrArray               *values,
+                                                        TrackerSparqlConnection *connection);
 
 G_DEFINE_ABSTRACT_TYPE (TrackerWritebackFile, tracker_writeback_file, TRACKER_TYPE_WRITEBACK)
 
@@ -60,9 +60,9 @@ file_unlock_cb (gpointer user_data)
 }
 
 static gboolean
-tracker_writeback_file_update_metadata (TrackerWriteback *writeback,
-                                        GPtrArray        *values,
-                                        TrackerClient    *client)
+tracker_writeback_file_update_metadata (TrackerWriteback        *writeback,
+                                        GPtrArray               *values,
+                                        TrackerSparqlConnection *connection)
 {
 	TrackerWritebackFileClass *writeback_file_class;
 	gboolean retval;
@@ -70,13 +70,11 @@ tracker_writeback_file_update_metadata (TrackerWriteback *writeback,
 	GFileInfo *file_info;
 	const gchar *urls[2] = { NULL, NULL };
 	GStrv row;
-	TrackerWritebackFile *self;
 	const gchar * const *content_types;
 	const gchar *mime_type;
 	guint n;
 
 	writeback_file_class = TRACKER_WRITEBACK_FILE_GET_CLASS (writeback);
-	self = TRACKER_WRITEBACK_FILE (writeback);
 
 	if (!writeback_file_class->update_file_metadata) {
 		g_critical ("%s doesn't implement update_file_metadata()",
@@ -90,11 +88,11 @@ tracker_writeback_file_update_metadata (TrackerWriteback *writeback,
 		return FALSE;
 	}
 
-	/* Get the file from the first row */
+	/* Get the file from the row */
 	row = g_ptr_array_index (values, 0);
 	file = g_file_new_for_uri (row[0]);
 
-	file_info = g_file_query_info (file, 
+	file_info = g_file_query_info (file,
 	                               G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
 	                               G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
 	                               NULL, NULL);
@@ -103,7 +101,6 @@ tracker_writeback_file_update_metadata (TrackerWriteback *writeback,
 		if (file) {
 			g_object_unref (file);
 		}
-
 		return FALSE;
 	}
 
@@ -133,7 +130,7 @@ tracker_writeback_file_update_metadata (TrackerWriteback *writeback,
 		                                          urls);
 
 		retval = (writeback_file_class->update_file_metadata) (TRACKER_WRITEBACK_FILE (writeback),
-		                                                       file, values, client);
+		                                                       file, values, connection);
 
 		g_timeout_add_seconds (3, file_unlock_cb, g_object_ref (file));
 	}
